@@ -20,7 +20,10 @@ use std::time::{Duration, SystemTime, UNIX_EPOCH};
 use clap::{App, Arg, SubCommand};
 use serde::{Deserialize, Serialize};
 
-use pandora_box::built_info;
+pub mod built_info {
+    // The file has been placed there by the build script.
+    include!(concat!(env!("OUT_DIR"), "/built.rs"));
+}
 
 #[derive(Copy, Clone, Debug, Eq, PartialEq, PartialOrd, Ord, Hash)]
 #[repr(u8)]
@@ -35,11 +38,11 @@ enum Sandbox {
 impl std::fmt::Display for Sandbox {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         match self {
-            Self::Bind => write!(f, "whitelist/network/bind"),
-            Self::Connect => write!(f, "whitelist/network/connect"),
-            Self::Write => write!(f, "whitelist/write"),
-            Self::Exec => write!(f, "whitelist/exec"),
-            Self::Read => write!(f, "#? whitelist/read"),
+            Self::Bind => write!(f, "allowlist/network/bind"),
+            Self::Connect => write!(f, "allowlist/network/connect"),
+            Self::Write => write!(f, "allowlist/write"),
+            Self::Exec => write!(f, "allowlist/exec"),
+            Self::Read => write!(f, "#? allowlist/read"),
         }
     }
 }
@@ -50,61 +53,66 @@ core/sandbox/read:off
 core/sandbox/write:deny
 core/sandbox/network:deny
 
-core/whitelist/per_process_directories:true
-core/whitelist/successful_bind:true
-core/whitelist/unsupported_socket_families:true
+core/allowlist/per_process_directories:true
+core/allowlist/successful_bind:true
+core/allowlist/unsupported_socket_families:true
 
 core/violation/decision:deny
 core/violation/exit_code:-1
 core/violation/raise_fail:false
 core/violation/raise_safe:false
 
-core/trace/follow_fork:true
 core/trace/magic_lock:off
-core/trace/use_seccomp:true
-core/trace/use_seize:true
 core/trace/use_toolong_hack:true
 
-core/restrict/file_control:false
+core/restrict/io_control:false
+core/restrict/memory_map:false
 core/restrict/shared_memory_writable:false
+core/restrict/general:0
 
 core/match/case_sensitive:true
 core/match/no_wildcard:prefix
 
-whitelist/write+/dev/stdout
-whitelist/write+/dev/stderr
-whitelist/write+/dev/zero
-whitelist/write+/dev/null
-whitelist/write+/dev/full
-whitelist/write+/dev/console
-whitelist/write+/dev/random
-whitelist/write+/dev/urandom
-whitelist/write+/dev/ptmx
-whitelist/write+/dev/fd/***
-whitelist/write+/dev/tty*
-whitelist/write+/dev/pty*
-whitelist/write+/dev/tts
-whitelist/write+/dev/pts
-whitelist/write+/dev/shm/***
-whitelist/write+/selinux/context/***
-whitelist/write+/proc/self/attr/***
-whitelist/write+/proc/self/fd/***
-whitelist/write+/proc/self/task/***
-whitelist/write+/tmp/***
-whitelist/write+/var/tmp/***
-whitelist/write+/var/cache/***
+allowlist/write+/dev/stdout
+allowlist/write+/dev/stderr
+allowlist/write+/dev/zero
+allowlist/write+/dev/null
+allowlist/write+/dev/full
+allowlist/write+/dev/console
+allowlist/write+/dev/random
+allowlist/write+/dev/urandom
+allowlist/write+/dev/ptmx
+allowlist/write+/dev/fd/***
+allowlist/write+/dev/tty*
+allowlist/write+/dev/pty*
+allowlist/write+/dev/tts
+allowlist/write+/dev/pts
+allowlist/write+/dev/shm/***
+allowlist/write+/selinux/context/***
+allowlist/write+/proc/self/attr/***
+allowlist/write+/proc/self/fd/***
+allowlist/write+/proc/self/task/***
+allowlist/write+/tmp/***
+allowlist/write+/var/tmp/***
+allowlist/write+/var/cache/***
 
-whitelist/network/bind+LOOPBACK@0
-whitelist/network/bind+LOOPBACK@1024-65535
-whitelist/network/bind+LOOPBACK6@0
-whitelist/network/bind+LOOPBACK6@1024-65535
+allowlist/network/bind+LOOPBACK@0
+allowlist/network/bind+LOOPBACK@1024-65535
+allowlist/network/bind+LOOPBACK6@0
+allowlist/network/bind+LOOPBACK6@1024-65535
 
-whitelist/network/connect+unix:/var/run/nscd/socket
-whitelist/network/connect+unix:/run/nscd/socket
-whitelist/network/connect+unix:/var/lib/sss/pipes/nss
+allowlist/network/connect+unix:/var/run/nscd/socket
+allowlist/network/connect+unix:/run/nscd/socket
+allowlist/network/connect+unix:/var/lib/sss/pipes/nss
 ";
 
-#[derive(Serialize, Deserialize, Clone, Debug)]
+#[derive(Serialize, Deserialize, Copy, Clone, Debug)]
+struct EventStruct {
+    id: u32,
+    //name: String,
+}
+
+#[derive(Serialize, Deserialize, Copy, Clone, Debug)]
 struct ProcessStruct {
     // pid: u32,
 // stat: StatStruct,
@@ -128,24 +136,33 @@ enum Dump {
     SysEnt {
         id: u32,
         ts: u64,
-        event: u16,
+        event: EventStruct,
         /*
-        event_name: String,
         pid: u32,
         ppid: u32,
         tgid: u32,
         */
-        sysname: String,
-        args: [u64; 6],
+        name: String,
+        args: [i64; 6],
         repr: [String; 6],
     },
+    ThreadNew {
+        id: u32,
+    },
+    ThreadFree {
+        id: u32,
+    },
 }
 
 fn command_box<'a>(
     bin: &'a str,
     cmd: &mut Vec<&'a str>,
+    arch: &Option<Vec<&'a str>>,
     config: &Option<Vec<&'a str>>,
     magic: &Option<Vec<&'a str>>,
+    bpf: bool,
+    dump: &Option<&'a str>,
+    export: &Option<&'a str>,
 ) -> i32 {
     cmd.insert(0, "--");
     if let Some(ref magic) = magic {
@@ -160,6 +177,23 @@ fn command_box<'a>(
             cmd.insert(0, "-c");
         }
     }
+    if let Some(ref arch) = arch {
+        for item in arch.iter() {
+            cmd.insert(0, item);
+            cmd.insert(0, "-a");
+        }
+    }
+    if bpf {
+        cmd.insert(0, "-b");
+    }
+    if let Some(dump_fd) = dump {
+        cmd.insert(0, dump_fd);
+        cmd.insert(0, "-d");
+    }
+    if let Some(export_format) = export {
+        cmd.insert(0, export_format);
+        cmd.insert(0, "--export");
+    }
     cmd.insert(0, bin);
     // eprintln!("executing `{:?}'", cmd);
     let cmdline: Vec<CString> = cmd
@@ -200,8 +234,6 @@ fn command_profile<'b>(bin: &'b str, cmd: &[&'b str], output_path: &'b str, path
         .arg("-m")
         .arg("core/sandbox/network:deny")
         .arg("-m")
-        .arg("core/restrict/file_control:0")
-        .arg("-m")
         .arg("core/restrict/shared_memory_writable:0")
         .arg("-d")
         .arg(format!("{}", fd_rw))
@@ -218,7 +250,7 @@ fn command_profile<'b>(bin: &'b str, cmd: &[&'b str], output_path: &'b str, path
 
     child.wait().expect("failed to wait for sydbox");
     eprintln!("success writing output to `{}' dump", output_path);
-    eprintln!("Edit the file ẁith your browser as necessary.");
+    eprintln!("Edit the file ẁith your editor as necessary.");
     eprintln!("Then use 'pandora box -c \"{}\" <command>'", output_path);
     eprintln!("To run the command under SydBox.");
 
@@ -231,6 +263,12 @@ fn command_inspect(input_path: &str, output_path: &str, path_limit: u8) -> i32 {
 }
 
 fn main() {
+    let arch_values = [
+        "native", "x86_64", "x86", "x32", "arm", "aarch64", "mips", "mips64", "ppc", "ppc64",
+        "ppc64le", "s390", "s390x", "parisc", "parisc64", "riscv64",
+    ];
+    let dump_values = ["fd[0-9]+", "path", "tmp"];
+    let export_values = ["bpf", "pfc"];
     let matches = App::new(built_info::PKG_NAME)
         .version(built_info::PKG_VERSION)
         .author(built_info::PKG_AUTHORS)
@@ -241,12 +279,12 @@ If no subcommands are given, Pandora executes a shell with the argument `-l'.
 To figure out the shell first the SHELL environment variable is checked.
 If this is not set, the default shell is `/bin/sh'.
 
-In login shell mode, if the file `/etc/pandora.syd-1' exists,
+In login shell mode, if the file `/etc/pandora.syd-2' exists,
 Pandora will tell SydBox to use this file as configuration.
 
 In login shell mode, SydBox uses the Paludis profile as the default set of configuration values.
 To see this default set of configuration values and white lists of system paths, check:
-https://git.exherbo.org/sydbox-1.git/plain/data/paludis.syd-1
+https://git.exherbo.org/sydbox-1.git/plain/data/paludis.syd-2
 
 Hey you, out there beyond the wall,
 Breaking bottles in the hall,
@@ -273,7 +311,6 @@ Repository: {}
                         .required(true)
                         .help("Path to sydbox binary")
                         .long("bin")
-                        .short("b")
                         .env("SYDBOX_BIN"),
                 )
                 .arg(
@@ -292,6 +329,50 @@ Repository: {}
                         .multiple(true)
                         .number_of_values(1),
                 )
+                .arg(
+                    Arg::with_name("arch")
+                        .default_value("native")
+                        .required(false)
+                        .help("filter system calls for the given architecture, may be repeated")
+                        .short("a")
+                        .long("arch")
+                        .number_of_values(1)
+                        .possible_values(&arch_values),
+                )
+                .arg(
+                    Arg::with_name("bpf-only")
+                        .required(false)
+                        .help("run in bpf only mode, no seccomp user notifications")
+                        .short("b"),
+                )
+                .arg(
+                    Arg::with_name("dump")
+                        .required(false)
+                        .help("dump system call information to the given file descriptor")
+                        .short("d")
+                        .number_of_values(1)
+                        .possible_values(&dump_values),
+                )
+                .arg(
+                    Arg::with_name("export")
+                        .required(false)
+                        .help("export the seccomp filters to standard error on startup")
+                        .long("export")
+                        .number_of_values(1)
+                        .possible_values(&export_values),
+                )
+                .arg(
+                    Arg::with_name("dry-run")
+                        .required(false)
+                        .help("run under inspection without denying system calls")
+                        .long("dry-run"),
+                )
+                .arg(
+                    Arg::with_name("test")
+                        .required(false)
+                        .help("test if various runtime requirements are functional and exit")
+                        .long("test"),
+                )
                 .arg(Arg::with_name("cmd").required(true).multiple(true)),
         )
         .subcommand(
@@ -303,12 +384,11 @@ Repository: {}
                         .required(true)
                         .help("Path to sydbox binary")
                         .long("bin")
-                        .short("b")
                         .env("SYDBOX_BIN"),
                 )
                 .arg(
                     Arg::with_name("output")
-                        .default_value("./out.syd-1")
+                        .default_value("./out.syd-2")
                         .required(true)
                         .help("Path to sydbox profile output")
                         .long("output")
@@ -339,7 +419,7 @@ Repository: {}
                 )
                 .arg(
                     Arg::with_name("output")
-                        .default_value("./out.syd-1")
+                        .default_value("./out.syd-2")
                         .required(true)
                         .help("Path to sydbox profile output")
                         .long("output")
@@ -359,10 +439,33 @@ Repository: {}
 
     if let Some(ref matches) = matches.subcommand_matches("box") {
         let bin = matches.value_of("bin").unwrap();
+        let bpf = matches.is_present("bpf-only");
         let mut cmd: Vec<&str> = matches.values_of("cmd").unwrap().collect();
+        let mut dump: Option<&str> = None;
+        if let Some(dump_fd) = matches.value_of("dump") {
+            dump = Some(dump_fd);
+        }
+        let mut export: Option<&str> = None;
+        if let Some(export_format) = matches.value_of("export") {
+            if export_format == "bpf" || export_format == "pfc" {
+                export = Some(export_format);
+            } else {
+                clap::Error::with_description(
+                    &format!(
+                        "Invalid value `{}' for --export: use bpf, pfc",
+                        export_format
+                    ),
+                    clap::ErrorKind::InvalidValue,
+                )
+                .exit();
+            }
+        }
+        let arch: Option<Vec<&str>> = matches.values_of("arch").map(|values| values.collect());
         let config: Option<Vec<&str>> = matches.values_of("config").map(|values| values.collect());
         let magic: Option<Vec<&str>> = matches.values_of("magic").map(|values| values.collect());
-        std::process::exit(command_box(bin, &mut cmd, &config, &magic));
+        std::process::exit(command_box(
+            bin, &mut cmd, &arch, &config, &magic, bpf, &dump, &export,
+        ));
     } else if let Some(ref matches) = matches.subcommand_matches("profile") {
         let bin = matches.value_of("bin").unwrap();
         let out = matches.value_of("output").unwrap();
@@ -405,7 +508,7 @@ Repository: {}
         let home;
         let mut homeargs = Vec::new();
         if let Ok(s) = std::env::var("HOME") {
-            home = format!("whitelist/write+{}/***", s);
+            home = format!("allowlist/write+{}/***", s);
             homeargs.push("-m");
             homeargs.push(&home);
         }
@@ -416,7 +519,7 @@ Repository: {}
             paludis.push(magic);
         }
 
-        let rcname = "/etc/pandora.syd-1";
+        let rcname = "/etc/pandora.syd-2";
         let rc = std::path::Path::new(rcname);
         let mut rcargs = Vec::new();
         if rc.exists() {
@@ -492,12 +595,13 @@ core/sandbox/network:deny
 
 # Further restrictions for open(), fcntl() and mmap()
 # See sydbox manual page for further details
-core/restrict/file_control:false
+core/restrict/io_control:false
+core/restrict/memory_map:false
 core/restrict/shared_memory_writable:false
 
-core/whitelist/per_process_directories:true
-core/whitelist/successful_bind:true
-core/whitelist/unsupported_socket_families:true
+core/allowlist/per_process_directories:true
+core/allowlist/successful_bind:true
+core/allowlist/unsupported_socket_families:true
 
 core/violation/decision:deny
 core/violation/exit_code:-1
@@ -513,41 +617,41 @@ core/match/case_sensitive:true
 core/match/no_wildcard:literal
 
 # Safe defaults for system paths
-whitelist/write+/dev/stdout
-whitelist/write+/dev/stderr
-whitelist/write+/dev/zero
-whitelist/write+/dev/null
-whitelist/write+/dev/full
-whitelist/write+/dev/console
-whitelist/write+/dev/random
-whitelist/write+/dev/urandom
-whitelist/write+/dev/ptmx
-whitelist/write+/dev/fd/***
-whitelist/write+/dev/tty*
-whitelist/write+/dev/pty*
-whitelist/write+/dev/tts
-whitelist/write+/dev/pts
-whitelist/write+/dev/pts/***
-whitelist/write+/dev/shm/***
-whitelist/write+/selinux/context/***
-whitelist/write+/proc/self/attr/***
-whitelist/write+/proc/self/fd/***
-whitelist/write+/proc/self/task/***
-whitelist/write+/tmp/***
-whitelist/write+/var/tmp/***
+allowlist/write+/dev/stdout
+allowlist/write+/dev/stderr
+allowlist/write+/dev/zero
+allowlist/write+/dev/null
+allowlist/write+/dev/full
+allowlist/write+/dev/console
+allowlist/write+/dev/random
+allowlist/write+/dev/urandom
+allowlist/write+/dev/ptmx
+allowlist/write+/dev/fd/***
+allowlist/write+/dev/tty*
+allowlist/write+/dev/pty*
+allowlist/write+/dev/tts
+allowlist/write+/dev/pts
+allowlist/write+/dev/pts/***
+allowlist/write+/dev/shm/***
+allowlist/write+/selinux/context/***
+allowlist/write+/proc/self/attr/***
+allowlist/write+/proc/self/fd/***
+allowlist/write+/proc/self/task/***
+allowlist/write+/tmp/***
+allowlist/write+/var/tmp/***
 
 # Safe defaults for local network
 # This allows bind to all loopback ports.
-# Each successful bind is automatically whitelisted for connect with
-# core/whitelist/successful_bind:true
-whitelist/network/bind+LOOPBACK@0
-whitelist/network/bind+LOOPBACK@1024-65535
-whitelist/network/bind+LOOPBACK6@0
-whitelist/network/bind+LOOPBACK6@1024-65535
+# Each successful bind is automatically allowlisted for connect with
+# core/allowlist/successful_bind:true
+allowlist/network/bind+LOOPBACK@0
+allowlist/network/bind+LOOPBACK@1024-65535
+allowlist/network/bind+LOOPBACK6@0
+allowlist/network/bind+LOOPBACK6@1024-65535
 
-whitelist/network/connect+unix:/var/run/nscd/socket
-whitelist/network/connect+unix:/run/nscd/socket
-whitelist/network/connect+unix:/var/lib/sss/pipes/nss
+allowlist/network/connect+unix:/var/run/nscd/socket
+allowlist/network/connect+unix:/run/nscd/socket
+allowlist/network/connect+unix:/var/lib/sss/pipes/nss
 ###
 
 ###
@@ -592,7 +696,7 @@ fn parse_json_line(
     path_limit: u8,
 ) -> (Option<String>, Option<String>, Option<SystemTime>) {
     match serde_json::from_str(&serialized)
-        .unwrap_or_else(|_| panic!("failed to parse `{}'", serialized))
+        .unwrap_or_else(|e| panic!("failed to parse `{}': {}", serialized, e))
     {
         Dump::Init {
             id: 0,
@@ -606,69 +710,71 @@ fn parse_json_line(
         Dump::StartUp { id: 1, cmd, ts, .. } => {
             return (None, Some(cmd), Some(UNIX_EPOCH + Duration::from_secs(ts)));
         }
+        Dump::ThreadNew { id: 5, .. } => {}
+        Dump::ThreadFree { id: 6, .. } => {}
         Dump::SysEnt {
-            event: 10,
+            event: EventStruct { id: 8, .. },
             repr,
-            sysname,
+            name,
             ..
-        } if sysname == "bind" => {
+        } if name == "bind" => {
             magic.insert((crate::Sandbox::Bind, repr[1].clone()));
         }
         Dump::SysEnt {
-            event: 10,
+            event: EventStruct { id: 8, .. },
             repr,
-            sysname,
+            name,
             ..
-        } if sysname == "connect" => {
+        } if name == "connect" => {
             magic.insert((crate::Sandbox::Connect, repr[1].clone()));
         }
         Dump::SysEnt {
-            event: 10,
+            event: EventStruct { id: 8, .. },
             repr,
-            sysname,
+            name,
             ..
-        } if sysname == "sendto" => {
+        } if name == "sendto" => {
             magic.insert((crate::Sandbox::Connect, repr[4].clone()));
         }
         Dump::SysEnt {
-            event: 10,
+            event: EventStruct { id: 8, .. },
             repr,
-            sysname,
+            name,
             ..
-        } if sysname == "execve" => {
+        } if name == "execve" => {
             magic.insert((crate::Sandbox::Exec, repr[0].clone()));
         }
         Dump::SysEnt {
-            event: 10,
+            event: EventStruct { id: 8, .. },
             args,
             repr,
-            sysname,
+            name,
             ..
         } => {
             let may_write: bool;
             let mut report_missing_handler = false;
             let mut repr_idx: [usize; 6] = [0; 6];
-            if sysname.ends_with("at") {
+            if name.ends_with("at") {
                 repr_idx[0] = 2;
             } else {
                 repr_idx[0] = 1;
             }
 
-            may_write = if sysname == "open" {
+            may_write = if name == "open" {
                 open_may_write(args[1])
-            } else if sysname == "openat" {
+            } else if name == "openat" {
                 open_may_write(args[2])
-            } else if sysname == "access" {
+            } else if name == "access" {
                 access_may_write(args[1])
-            } else if sysname == "faccessat" {
+            } else if name == "faccessat" {
                 access_may_write(args[2])
-            } else if sysname == "rename" {
+            } else if name == "rename" {
                 repr_idx[1] = 2;
                 true
-            } else if sysname == "symlink" {
+            } else if name == "symlink" {
                 repr_idx[0] = 2;
                 true
-            } else if sysname == "mkdir" || sysname == "rmdir" || sysname == "unlink" {
+            } else if name == "mkdir" || name == "rmdir" || name == "unlink" {
                 true
             } else {
                 report_missing_handler = true;
@@ -676,7 +782,7 @@ fn parse_json_line(
             };
 
             if report_missing_handler {
-                eprintln!("SYS:{:?} {:?} {:?}", sysname, args, repr);
+                eprintln!("SYS:{:?} {:?} {:?}", name, args, repr);
             }
 
             for idx in &repr_idx {
@@ -756,11 +862,11 @@ fn filter_proc(path: &str) -> String {
     path.to_string()
 }
 
-fn access_may_write(mode: u64) -> bool {
+fn access_may_write(mode: i64) -> bool {
     (mode as i32) & libc::W_OK != 0
 }
 
-fn open_may_write(flags: u64) -> bool {
+fn open_may_write(flags: i64) -> bool {
     let flags: i32 = flags as i32;
     match flags & libc::O_ACCMODE {
         libc::O_WRONLY | libc::O_RDWR => true,
